'#########################################################################
'#  Termometr "wierkajcy" alfabetem Morsea                            #
'#  Mikrokontroler: ATTiny13A, wewntrzny RC 9.6 MHz, Enable DIV8=1.2MHz #
'#  Plik rdowy: MTHR_0v2.bas                                          #
'#  Data ostatniej modyfikacji: 26.11.2014                               #
'#  Autor: c4v2 Cyprian Kamil Kowalski                                   #
'#########################################################################
'+-------------------------+
'| Przypisania pinw:      |
'+------+--------+---------+
'| PIN  |  I/O   | Funkcja |
'+------+--------+---------+
'| PB.0 |  OUT   | LED     |
'| PB.1 |  OUT   | BUZZ    |
'| PB.2 |  OUT   | 1WPU    |
'| PB.3 | IN/OUT | DSDQ    |
'| PB.4 |  IN    | MODE    | Z Kod skrcony  L Kod zwyky
'| PB.5 | RESET  | WAKEUP  | Wczenie pomiaru, wygenerowanie kodu i przecie w stan upienia
'+------+--------+---------+
'+-------------------------------------------------+
'| Dwa pierwsze bajty dla DS1820 (DS_Type=0):      |
'+-------------+-----------------------+-----------+
'| warto     |       binarnie        |    hex    |
'| temperatury |    MSB    |    LSB    | MSB | LSB |
'|             | **** **** | tttt ttt- |     |     |
'| +125C      | 0000 0000 | 1111 1010 | 00  | FA  |
'| +25C       | 0000 0000 | 0011 0010 | 00  | 32  |
'| +1/2C      | 0000 0000 | 0000 0001 | 00  | 01  |
'| +0C        | 0000 0000 | 0000 0000 | 00  | 00  |
'| 1/2C      | 1111 1111 | 1111 1111 | FF  | FF  |
'| 25C       | 1111 1111 | 1100 1110 | FF  | CE  |
'| 55C       | 1111 1111 | 1001 0010 | FF  | 92  |
'+-------------+-----------+----------+------+-----+
'+-------------------------------------------------+
'| Dwa pierwsze bajty dla DS18B20 (DS_Type=1):     |
'+-------------+-----------------------+-----------+
'| warto     |       binarnie        |    hex    |
'| temperatury |    MSB    |    LSB    | MSB | LSB |
'|             | **** *ttt | tttt ---- |     |     |
'| + 125C     | 0000 0111 | 1101 0000 | 07  | D0  |
'| + 85C*     | 0000 0101 | 0101 0000 | 05  | 50  |
'| + 25.0625C | 0000 0001 | 1001 0001 | 01  | 91  |
'| + 10.125C  | 0000 0000 | 1010 0010 | 00  | A2  |
'| + 0.5C     | 0000 0000 | 0000 1000 | 00  | 08  |
'| 0C         | 0000 0000 | 0000 0000 | 00  | 00  |
'| -0.5C      | 1111 1111 | 1111 1000 | FF  | F8  |
'| -10.125C   | 1111 1111 | 0101 1110 | FF  | 5E  |
'| -25.0625C  | 1111 1110 | 0110 1111 | FE  | 6F  |
'| -55C       | 1111 1100 | 1001 0000 | FC  | 90  |
'+-------------+-----------+-----------+-----+-----+
'----------------------------------------------------------------------------------------
'$Sim
$regfile = "attiny13a.dat"
$crystal = 1200000                      '9600000/DIV8
$hwstack = 24
$swstack = 0
$framesize = 16
$noinit
$noramclear
'$map

'----------------------------------------------------------------------------------------
Const Freq = 750                        'Czstotliwo tonu (LED) [ Hz ]
Const Dot_ms = 55                       'Interwa kropki [ ms ]
Const DS_Type = 0                       'Typ czujnika: [ 0 ]=DS18S20, [ 1 ]=DS18B20
'----------------------------------------------------------------------------------------
Const Dash_ms = Dot_ms * 3
Const Pause_ms = Dot_ms * 2
Const Oc0A = _XTAL \ 8 \ 2 \ Freq
Dim NbChr As Byte
Dim Code As Byte
Dim DsData(2) As Byte
Dim CodeTab(3) As Byte

'----------------------------------------------------------------------------------------
$asm
  SBI ACSR , ACD    ; Wyaczenie zasilania komparatora analogowego
  SBI PRR , PRADC   ; Wyaczenie zasilania bloku przetwornika AD
  LDI R16 , &B00000111
  OUT Ddrb , R16
  LDI R16 , &B00011100
  OUT PORTB , R16
$end asm
Config 1wire = PORTB.3
1wreset                                 'Reset magistrali 1wire.
1wwrite &HCC                            'Wysanie rozkazu pomijania odczytu ROM czujnika (jeden czujnik slave na magistrali 1Wire).
1wwrite &H44                            'Wysanie rozkazu inicjujcego konwersj temperatury.
Waitms 750                              'Oczekiwanie na konwersj.
1wreset                                 'Reset magistrali 1wire.
1wwrite &HCC                            'Wysanie rozkazu pomijania odczytu ROM czujnika (jeden czujnik slave na magistrali 1Wire)
1wwrite &HBE                            'Wysanie rozkazu odczytu zmierzonej temperatury (9 bajtw patrz tabela wyzej)
DsData(1) = 1wread(2)                   'Odczytanie dwch bajtw (dwch, poniewa nie sprawdzana jest suma kontrolna) temperatury z magistrali 1Wire.
1wreset
$asm
  CBI DDRB , DDB2    ; Wyczenie podcigania magistrali 1Wire
  LDS R16 , {Err}
  CPI R16 , $00
  BREQ 07_1WOK
  RJMP 07_SLEEP
07_1WOK:
  LDI R18 , $04      ; Zakadana wstpna liczba znakw = n-1
  LDS R16 , {DsData}
  LDS R17 , {DsData+1}
$end asm
#If DS_Type = 0                         'Konwersja dla DS18S20:
  $asm
    SBRS R17 , 7
    RJMP 00_PLUS   ; Jeeli temperatura dodatnia pominicie instrukcji do etykiety 00_PLUS
    DEC R18        ; liczba znakw <=3
    NEG R16        ; Obliczenie wartoci absolutnej ze zmiennej w kodzie U2
  00_PLUS:
    LSR R16
    STS {DsData} , R16
  $end asm
#EndIf
#If DS_Type = 1    'Konwersja dla DS18B20:
  $asm
    SWAP R16
    ANDI R16 , $0F
    SWAP R17
    ANDI R17 , $F0
    OR R16 , R17
    SBRS R16 , 7
    RJMP 01_PLUS   ; Jeeli temperatura dodatnia pominicie instrukcji do etykiety 01_PLUS
    DEC R18        ; liczba znakw <=3
    NEG R16        ; Obliczenie wartoci absolutnej ze zmiennej w kodzie U2
  01_PLUS:
    STS {DsData} , R16
  $end asm
#EndIf
$asm
  LDI R17 , $0A         ; Indeks znaku "minus"
  STS {CodeTab+1} , R17
  DEC R18               ; liczba znakw <=3
  CPI R16 , $0A
  BRLO 02_ooN
  STS {CodeTab} , R17
  DEC R18               ; liczba znakw <=2
  CPI R16 , $64
  BRLO 02_oNN
  DEC R18               ; liczba znakw =1
  CPI R16 , $63
  BRSH 02_NNN
$end asm
02_NNN:
  CodeTab(1) = DsData(1) \ 100
02_oNN:
  CodeTab(2) = DsData(1) mod 100
  CodeTab(2) = CodeTab(2) \ 10
02_ooN:
  CodeTab(3) = DsData(1) mod 10
$asm
  LDI R16 , Oc0A
  OUT OCR0A , R16
03_LOOP:
  STS {NbChr} , R18
  SBIC PINB , 4
  RJMP 04_LONG
$end asm
  Code = Lookup(CodeTab(NbChr) , Short_Morsea)
  RJMP 04_END
04_LONG:
  Code = Lookup(CodeTab(NbChr) , Long_Morsea)
04_END:
$asm
  LDS R16 , {Code}
05_LOOP:
  LDI R17 , $42
  OUT TCCR0A , R17 ; Tryb CTC , zmiana stanu wyjcia OC0A przy zrwnaniu z OCR0A
  LDI R17 , $00
  OUT TCNT0 , R17  ; Licznik liczy od zera
  LDI R17 , $82
  OUT TCCR0B , R17 ; Wczenie licznika generatora tonu na OC0A , Preskaler = 8
  SBI PORTB , 1    ; Ustawienie stanu H na nce PB.1 na czas:
  SBRC R16 , 7     ; zalenie od 7 bitu R16
  RJMP 06_MDOT
$end asm
  Waitms Dash_ms                        'Kreski
  RJMP 06_END
06_MDOT:
  Waitms Dot_ms                         'Kropki
06_END:
$asm
  LDI R17 , $80
  OUT TCCR0B , R17 ; Wyczenie licznika generatora tonu
  LDI R17 , $00
  OUT TCCR0A , R17 ; Wyaczenie alternatywnej funkcji OC1A PB.0
  CBI PORTB , 1    ; Ustawienie stanu L na nce PB.1
  CBI PORTB , 0    ; Ustawienie stanu L na nce PB.0
$end asm
Waitms Dot_ms                           'Czas midzy impulsami w znaku = [.]
$asm
  LSL R16          ; "Wysunicie" ju zinterpretowanego bitu
  CPI R16 , $80    ; Warto &B10000000 oznacza koniec nadawania znaku
  BRNE 05_LOOP     ; Jeeli warto inna to nadanie kolejnego impulsu
$end asm
Waitms Pause_ms    'Wyduenie "czasu ciszy" dla uzyskania interwau midzy znakami = [...]
$asm
  INC R18
  CPI R18 , $04    ; Nadano wszystkie znaki
  BRNE 03_LOOP
07_SLEEP:
  LDI R16 , $00
  OUT PORTB , R16
  LDI R16 , $FF
  OUT DDRB , R16
  SBI PRR , PRTIM0  ; Wyaczenie zasilania bloku licznika T/C0
  SBI MCUCR , SM1   ; Wybr trybu stanu niskiego poboru energi
  SBI MCUCR , SE    ; Zezwolenie na przejcie w stan niskiego poboru energi
  SLEEP             ; Przejcie mikrokontrolera w stan niskiego poboru energi
$end asm
End

'----------------------------------------------------------------------------------------
Long_Morsea:
  Data &B00000100                       '0      
  Data &B10000100                       '1      
  Data &B11000100                       '2      
  Data &B11100100                       '3      
  Data &B11110100                       '4      
  Data &B11111100                       '5      
  Data &B01111100                       '6      
  Data &B00111100                       '7      
  Data &B00011100                       '8      
  Data &B00001100                       '9      
  Data &B01111010                       '       

Short_Morsea:
  Data &B01000000                       '0  
  Data &B10100000                       '1   
  Data &B11010000                       '2    
  Data &B11101000                       '3     
  Data &B11110100                       '4      
  Data &B11111100                       '5      
  Data &B01111100                       '6      
  Data &B01111000                       '7     
  Data &B01110000                       '8    
  Data &B01100000                       '9   
  Data &B01111010                       '       

'ENDE------------------------------------------------------------------------------------